﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;

namespace Win
{
    /// <summary> 
    /// 网络通讯事件模型委托 
    /// </summary> 
    public delegate void NetEvent(object sender, NetEventArgs e);

    public class TcpClient 
    {
        public event WraperEventHandler WraperEvent;
        public delegate void WraperEventHandler(Object sender, OperType eventenum);

                //定义操作类型 读写 和 为止 
        public enum OperType
        {
            ConnectOK,
            ConnectErr, //打开串口错误 
            Close,    //关闭
            CloseErr,  //关闭错误
            UnknownErr,  //未知错误

            Rec,       //接收数据
            Send,      //发送数据
            SendErr,   //发送数据错误
            RecErr,    //接收数据错误

            Connected,
        }

        #region

        public  OperType operSort ;

        /// <summary> 
        /// 服务器程序使用的端口 
        /// </summary> 
        private ushort _Serverport;

        /// <summary> 
        /// 服务器程序绑定的IP
        /// </summary> 
        private IPAddress _ServerIP;

        /// <summary> 
        /// 末次错误信息
        /// </summary> 
        private string _LastErr;

        /// <summary> 
        /// 运行状态 
        /// </summary> 
        private bool _isConnect;

        /// <summary> 
        /// 服务器使用的异步Socket类, 
        /// </summary> 
        private Socket _svrSock;

        /// <summary> 
        /// 接收数据缓冲区 
        /// </summary> 
        private byte[] _recvDataBuffer;


        /// <summary> 
        /// 服务器的Socket对象 
        /// </summary> 
        public Socket ServerSocket
        {
            get { return _svrSock; }
        }

         /// <summary> 
        /// 末次错误信息
        /// </summary> 
        public string LastErr
        {
            get { return _LastErr; }
        }

        

        ///// <summary> 
        ///// 数据报文分析器 
        ///// </summary> 
        //public DatagramResolver Resovlver
        //{
        //    get { return _resolver; }
        //    set { _resolver = value; }
        //}

        /// <summary> 
        /// 服务器运行状态 
        /// </summary> 
        public bool IsConnet
        {
            get { return _isConnect; }
        }

        #endregion


        #region 事件定义

        /// <summary> 
        /// 客户端建立连接事件 
        /// </summary> 
        public event NetEvent ClientConn;

        /// <summary> 
        /// 客户端关闭事件 
        /// </summary> 
        public event NetEvent ClientClose;

        /// <summary> 
        /// 服务器接收到数据事件 
        /// </summary> 
        public event NetEvent RecvData;

        #endregion

        /// <summary>
        /// 构造函数 定义服务器IP 和 端口
        /// </summary>
        /// <param name="Ip"></param>
        /// <param name="Port"></param>
        public TcpClient()
        {           
            _Serverport = 2020;
            _ServerIP =IPAddress.Parse("127.0.0.1");

        }
        public void TcpClientcfg(IPAddress Ip, ushort Port)
        {
            _Serverport = Port;
            _ServerIP = Ip;
        }
        public void ConnectServer()
        {
            try
            {
                _svrSock = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                _svrSock.Connect(this._ServerIP, _Serverport);	//建立连接.
                OnWraperEvent(OperType.ConnectOK);
                _isConnect=true ;
            }
            catch (Exception ex)
            {
                _LastErr = ex.Message;
                OnWraperEvent(OperType.ConnectErr);
            }

        }
         /// <summary> 
        /// 客户端连接处理函数 
        /// </summary> 
        /// <param name="iar">欲建立服务器连接的Socket对象</param> 
        protected virtual void CConnect(Socket _svrSock)
        {

        }

        /// <summary>
        /// 断开连接
        /// </summary>
          public void disConnect()
          {
              try
              {
                  if (_svrSock.Connected )
                  {
                      _svrSock.Close();
                  }
              }
              catch ( Exception ex)
              {
                  _LastErr = ex.Message;
                  OnWraperEvent(OperType.CloseErr);
              }
          }
        
        // <summary>

        /// <summary>
        /// 发送事件
        /// </summary>
        /// <param name="eventenum"></param>
        protected virtual void OnWraperEvent(OperType eventenum)
        {
            if (this.WraperEvent != null)
            {
                WraperEvent(this, eventenum);
            }
        }
    }

    /// <summary> 
    /// 服务器程序的事件参数,包含了激发该事件的会话对象 
    /// </summary> 
    public class NetEventArgs : EventArgs
    {

        #region 字段

        /// <summary> 
        /// 客户端与服务器之间的会话 
        /// </summary> 
        private Session _client;

        #endregion

        #region 构造函数
        /// <summary> 
        /// 构造函数 
        /// </summary> 
        /// <param name="client">客户端会话</param> 
        public NetEventArgs(Session client)
        {
            if (null == client)
            {
                throw (new ArgumentNullException());
            }

            _client = client;
        }
        #endregion

        #region 属性

        /// <summary> 
        /// 获得激发该事件的会话对象 
        /// </summary> 
        public Session Client
        {
            get
            {
                return _client;
            }

        }

        #endregion
    }

    /// <summary> 
    /// 客户端与服务器之间的会话类 
    /// 
    /// 版本: 1.1 
    /// 替换版本: 1.0 
    /// 
    /// 说明: 
    /// 会话类包含远程通讯端的状态,这些状态包括Socket,报文内容, 
    /// 客户端退出的类型(正常关闭,强制退出两种类型) 
    /// </summary> 
    public class Session : ICloneable
    {
        #region 字段

        /// <summary> 
        /// 会话ID 
        /// </summary> 
        private SessionId _id;

        /// <summary> 
        /// 客户端发送到服务器的报文 
        /// 注意:在有些情况下报文可能只是报文的片断而不完整 
        /// </summary> 
        private string _datagram;

        /// <summary> 
        /// 客户端的Socket 
        /// </summary> 
        private Socket _cliSock;

        /// <summary> 
        /// 客户端的退出类型 
        /// </summary> 
        private ExitType _exitType;

        /// <summary> 
        /// 退出类型枚举 
        /// </summary> 
        public enum ExitType
        {
            NormalExit,
            ExceptionExit
        };

        #endregion

        #region 属性

        /// <summary> 
        /// 返回会话的ID 
        /// </summary> 
        public SessionId ID
        {
            get
            {
                return _id;
            }
        }

        /// <summary> 
        /// 存取会话的报文 
        /// </summary> 
        public string Datagram
        {
            get
            {
                return _datagram;
            }
            set
            {
                _datagram = value;
            }
        }

        /// <summary> 
        /// 获得与客户端会话关联的Socket对象 
        /// </summary> 
        public Socket ClientSocket
        {
            get
            {
                return _cliSock;
            }
        }

        /// <summary> 
        /// 存取客户端的退出方式 
        /// </summary> 
        public ExitType TypeOfExit
        {
            get
            {
                return _exitType;
            }

            set
            {
                _exitType = value;
            }
        }

        #endregion

        #region 方法

        /// <summary> 
        /// 使用Socket对象的Handle值作为HashCode,它具有良好的线性特征. 
        /// </summary> 
        /// <returns></returns> 
        public override int GetHashCode()
        {
            return (int)_cliSock.Handle;
        }

        /// <summary> 
        /// 返回两个Session是否代表同一个客户端 
        /// </summary> 
        /// <param name="obj"></param> 
        /// <returns></returns> 
        public override bool Equals(object obj)
        {
            Session rightObj = (Session)obj;

            return (int)_cliSock.Handle == (int)rightObj.ClientSocket.Handle;

        }

        /// <summary> 
        /// 重载ToString()方法,返回Session对象的特征 
        /// </summary> 
        /// <returns></returns> 
        public override string ToString()
        {
            string result = string.Format("Session:{0},IP:{1}",
            _id, _cliSock.RemoteEndPoint.ToString());

            //result.C 
            return result;
        }

        /// <summary> 
        /// 构造函数 
        /// </summary> 
        /// <param name="cliSock">会话使用的Socket连接</param> 
        public Session(Socket cliSock)
        {
            Debug.Assert(cliSock != null);

            _cliSock = cliSock;

            _id = new SessionId((int)cliSock.Handle);
        }

        /// <summary> 
        /// 关闭会话 
        /// </summary> 
        public void Close()
        {
            Debug.Assert(_cliSock != null);

            //关闭数据的接受和发送 
            _cliSock.Shutdown(SocketShutdown.Both);

            //清理资源 
            _cliSock.Close();
        }

        #endregion

        #region ICloneable 成员

        object System.ICloneable.Clone()
        {
            Session newSession = new Session(_cliSock);
            newSession.Datagram = _datagram;
            newSession.TypeOfExit = _exitType;

            return newSession;
        }

        #endregion
    }

    /// <summary> 
    /// 唯一的标志一个Session,辅助Session对象在Hash表中完成特定功能 
    /// </summary> 
    public class SessionId
    {
        /// <summary> 
        /// 与Session对象的Socket对象的Handle值相同,必须用这个值来初始化它 
        /// </summary> 
        private int _id;

        /// <summary> 
        /// 返回ID值 
        /// </summary> 
        public int ID
        {
            get
            {
                return _id;
            }
        }

        /// <summary> 
        /// 构造函数 
        /// </summary> 
        /// <param name="id">Socket的Handle值</param> 
        public SessionId(int id)
        {
            _id = id;
        }

        /// <summary> 
        /// 重载.为了符合Hashtable键值特征 
        /// </summary> 
        /// <param name="obj"></param> 
        /// <returns></returns> 
        public override bool Equals(object obj)
        {
            if (obj != null)
            {
                SessionId right = (SessionId)obj;

                return _id == right._id;
            }
            else if (this == null)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary> 
        /// 重载.为了符合Hashtable键值特征 
        /// </summary> 
        /// <returns></returns> 
        public override int GetHashCode()
        {
            return _id;
        }

        /// <summary> 
        /// 重载,为了方便显示输出 
        /// </summary> 
        /// <returns></returns> 
        public override string ToString()
        {
            return _id.ToString();
        }

    } 

}



//private ClntSocket _clntSocket = null;       
//public ClientWinMain()         
//{            
//InitializeComponent();        
//}        
//private void btn_conn_server_Click(object sender, EventArgs e)         
//{             
//string ip = this.txt_ip.Text.Trim();            
//this._clntSocket = new ClntSocket(ip, 8989);             
//this._clntSocket.Connected  = new NetEventHandler(_clntSocket_Connected);             
//this._clntSocket.ConntentFailed  = new NetEventHandler(_clntSocket_ConntentFailed);             
//this._clntSocket.LoginSuccessed  = new NetEventHandler(_clntSocket_LoginSuccessed);             
//this._clntSocket.LoginFailed  = new NetEventHandler(_clntSocket_LoginFailed);             
//this._clntSocket.RecvUserList  = new NetEventHandler(_clntSocket_RecvUserList);             
//this._clntSocket.RecvMessage  = new NetEventHandler(_clntSocket_RecvMessage);            
//this._clntSocket.ServerClosed  = new NetEventHandler(_clntSocket_ServerClosed);            
//this._clntSocket.UserLogout  = new NetEventHandler(_clntSocket_UserLogout);             
//this._clntSocket.StartConn();         }           
//void _clntSocket_UserLogout(object sender, NetEventArgs e)         
//{             
////用户注销            
//this.txt_username.Enabled = true;            
//this.txt_pwd.Enabled = true;            
//this.btn_login.Enabled = true;             
//this.btn_logout.Enabled = false;             
//this.gb_msg.Enabled = false;         
//}           
///// <summary>         
///// 接收到信息         
///// </summary>         
///// <param name="sender">this._clntSocket对象</param>         
///// <param name="e">MsgData对象</param>         
//void _clntSocket_RecvMessage(object sender, NetEventArgs e)         
//{             
//MsgData msgData = (MsgData)e.EventArg;             
//if (msgData.IsBroadcast)             
//{                 
//if (this._clntSocket.UserData.UserName == msgData.Sender)                    
//this.txt_msg_list.AppendText("[我]对[所有人]说："   msgData.Msg   "\r\n");               
//else                    
//this.txt_msg_list.AppendText("["   msgData.Sender   "]对[所有人]说："   msgData.Msg   "\r\n");         
//}            
//else            
//this.txt_msg_list.AppendText("["   msgData.Sender   "]对[你]说："   msgData.Msg   "\r\n");     
//}
         
///// <summary>        
///// /// 取得用户列表         
///// </summary>         
///// <param name="sender">this._clntSocket对象</param>         
///// <param name="e">用户列表e.EventArg 是 StringCollection</param>        
//void _clntSocket_RecvUserList(object sender, NetEventArgs e)         
//{             
//StringCollection userList = (StringCollection)e.EventArg;            
//this.cbb_user_list.Items.Clear();            
//foreach (string tmp in userList)             
//{                 
//if(this._clntSocket.UserData.UserName != tmp)//不把自己的名字加入列表                   
//cbb_user_list.Items.Add(tmp);             }            
//if (this.cbb_user_list.Items.Count > 0)               
//this.cbb_user_list.SelectedIndex = 0;            
//else               
//this.cbb_user_list.Update();       
//}         
//void _clntSocket_LoginFailed(object sender, NetEventArgs e)       
//{           
//MessageBox.Show(e.EventArg.ToString());//简单显示一下信息      
//}     
//void _clntSocket_ServerClosed(object sender, NetEventArgs e)      
//{            
////服务器关闭回调           
//MessageBox.Show(e.EventArg.ToString());         }        
//void _clntSocket_LoginSuccessed(object sender, NetEventArgs e)       
//{            
////登录成功           
//this.txt_username.Enabled = false;           
//this.txt_pwd.Enabled = false;            
//this.btn_login.Enabled = false;             
//this.btn_logout.Enabled = true;          
//this.gb_msg.Enabled = true;        
//}           
//void _clntSocket_ConntentFailed(object sender, NetEventArgs e)     
//{           
//MessageBox.Show("链接失败！");     
//}          
//void _clntSocket_Connected(object sender, NetEventArgs e)      
//{             
////连接成功对一系列的空间进行设置         
//this.btn_conn_server.Enabled = false;       
//this.btn_dis_conn.Enabled = true;        
//this.gb_login.Enabled = true;        
//}           
//private void btn_dis_conn_Click(object sender, EventArgs e)   
//{            
//if (_clntSocket != null)          
//_clntSocket.Stop();            
//this.btn_conn_server.Enabled = true;      
//this.btn_dis_conn.Enabled = false;            
//this.gb_login.Enabled = false;           
//this.gb_msg.Enabled = false;             
//this.txt_username.Enabled = true;       
//this.txt_pwd.Enabled = true;           
//this.btn_login.Enabled = true;        
//this.btn_logout.Enabled = false;    
//}         
//private void btn_login_Click(object sender, EventArgs e)   
//{           
//if (this._clntSocket != null)           
//{               
//PacketHeaderStruct phs = new PacketHeaderStruct();  
//phs.type = StructConfig.GetHeaderIndexByType(HeaderType.UserLogin);     
//UserData ud = new UserData();               
//ud.UserName = this.txt_username.Text.Trim();      
//ud.UserPwd = this.txt_pwd.Text;
////密码可能有空格，所以不做缩空处理          
//byte[] data = SerializeTools.ObjSerialize(ud);        
//phs.length = data.Length;              
//this._clntSocket.Send(phs);            
//this._clntSocket.Send(data);          
////保存用户                 
//this._clntSocket.UserData = ud;       
//}             
//else           
//{                
//MessageBox.Show("clntSocket 空");          
//} 
//}        
//private void btn_logout_Click(object sender, EventArgs e)    
//{             //构造信息头             
//PacketHeaderStruct phs = new PacketHeaderStruct();      
//phs.type = StructConfig.GetHeaderIndexByType(HeaderType.UserLogout);    
//phs.length = 0;          
//this._clntSocket.Send(phs);        
//}         
//private void rbtn_one_Click(object sender, EventArgs e)   
//{            
//this.rbtn_all.Checked = false;    
//this.rbtn_one.Checked = true;          
//this.cbb_user_list.Enabled = true;   
//}          
//private void rbtn_all_Click(object sender, EventArgs e)      
//{          
//this.rbtn_all.Checked = true;      
//this.rbtn_one.Checked = false; 
//this.cbb_user_list.Enabled = false;     
//}           
//private void btn_send_Click(object sender, EventArgs e)   
//{            
//if (this._clntSocket != null && this._clntSocket.IsConnected)      
//{               
////构造信息体             
//MsgData msgData = new MsgData();            
//msgData.Msg = this.txt_send.Text.Trim();    
//if (this.rbtn_all.Checked)               
//{                   
//msgData.IsBroadcast = true;         
//}               
//else            
//{                   
//msgData.IsBroadcast = false;             
//msgData.Target = this.cbb_user_list.SelectedItem.ToString();     
//}                
//byte[] msgBin = SerializeTools.ObjSerialize(msgData);      
////构造信息头               
//PacketHeaderStruct phs = new PacketHeaderStruct();             
//phs.type = StructConfig.GetHeaderIndexByType(HeaderType.SendMessage);     
//phs.length = msgBin.Length;              
////发送                
//this._clntSocket.Send(phs);        
//this._clntSocket.Send(msgBin);     
////加入一条信息到文本框            
//if (this.rbtn_one.Checked)             
//{                    
//this.txt_msg_list.AppendText("[我]对[" this.cbb_user_list.Text "]说："   msgData.Msg   "\r\n");    
//}                 this.txt_send.Text = "";            
//}       
//}         
//private void btn_test_Click(object sender, EventArgs e)        
//{             MessageBox.Show(this.cbb_user_list.Text);      
//}        
//private void ClientWinMain_FormClosed(object sender, FormClosedEventArgs e)    
//{            
//if (this._clntSocket.IsConnected)    
//{                 this._clntSocket.Stop();       
//}         
//}           
//private void txt_send_KeyPress(object sender, KeyPressEventArgs e)         
//{             TextBox inputBox = sender as TextBox;         
//if(e.KeyChar=='\r')            
//{              
//e.Handled = true;//表示已经处理过按键，目的：取消“滴”的声音   
//this.btn_send_Click(sender, e);          
//}    
//}    
//} 
//}





